package logic

import (
	"context"
	"encoding/json"
	"fusenapi/model/gmodel"
	"fusenapi/server/product-template-tag/internal/svc"
	"fusenapi/server/product-template-tag/internal/types"
	"fusenapi/service/repositories"
	"fusenapi/utils/auth"
	"fusenapi/utils/basic"
	"fusenapi/utils/s3url_to_s3id"

	"github.com/zeromicro/go-zero/core/logx"
)

type GetProductTemplateTagsLogic struct {
	logx.Logger
	ctx    context.Context
	svcCtx *svc.ServiceContext
}

func NewGetProductTemplateTagsLogic(ctx context.Context, svcCtx *svc.ServiceContext) *GetProductTemplateTagsLogic {
	return &GetProductTemplateTagsLogic{
		Logger: logx.WithContext(ctx),
		ctx:    ctx,
		svcCtx: svcCtx,
	}
}

// 处理进入前逻辑w,r
// func (l *GetProductTemplateTagsLogic) BeforeLogic(w http.ResponseWriter, r *http.Request) {
// }

// 处理逻辑后 w,r 如:重定向, resp 必须重新处理
// func (l *GetProductTemplateTagsLogic) AfterLogic(w http.ResponseWriter, r *http.Request, resp *basic.Response) {
// // httpx.OkJsonCtx(r.Context(), w, resp)
// }
type logoSelect struct {
	LogoSelected struct {
		LogoSelectedId      int `json:"logo_selected_id"`
		TemplateTagSelected struct {
			TemplateTag   string     `json:"template_tag"`
			Color         [][]string `json:"color"`
			SelectedIndex int        `json:"selected_index"`
		} `json:"template_tag_selected"`
	} `json:"logo_selected"`
}

func (l *GetProductTemplateTagsLogic) GetProductTemplateTags(req *types.GetProductTemplateTagsReq, userinfo *auth.UserInfo) (resp *basic.Response) {
	userinfo.UserId = 127
	if req.Limit <= 0 || req.Limit > 100 {
		req.Limit = 5
	}
	var (
		productTemplateTags []gmodel.FsProductTemplateTags
		err                 error
	)
	//获取用户需要渲染logo
	logoInfo, err := l.svcCtx.Repositories.ImageHandle.LogoInfo(l.ctx, &repositories.LogoInfoReq{
		UserId:  userinfo.UserId,
		GuestId: userinfo.GuestId,
	})
	if err != nil {
		logx.Error(err)
		return
	}
	if logoInfo.Metadata == nil || *logoInfo.Metadata == "" {
		return resp.SetStatusWithMessage(basic.CodeDbSqlErr, "logo info`s metadata is not set")
	}
	//解析元数据
	var metaData map[string]interface{}
	if err = json.Unmarshal([]byte(*logoInfo.Metadata), &metaData); err != nil {
		logx.Error(err)
		return resp.SetStatusWithMessage(basic.CodeJsonErr, "failed to parse user metadata")
	}
	var mapTemplateTag map[string][][]string
	b, _ := json.Marshal(metaData["template_tag"])
	if err = json.Unmarshal(b, &mapTemplateTag); err != nil {
		logx.Error(err)
		return resp.SetStatusWithMessage(basic.CodeJsonErr, "invalid format of metadata`s template_tag")
	}
	//获取选中的颜色数据
	mapSelectColor := make(map[string]int) //key是模板标签val是选中的索引
	if logoInfo.UserInfoMetadata != nil && *logoInfo.UserInfoMetadata != "" {
		var logoSelectInfo logoSelect
		if err = json.Unmarshal([]byte(*logoInfo.UserInfoMetadata), &logoSelectInfo); err != nil {
			logx.Error(err)
			return resp.SetStatusWithMessage(basic.CodeJsonErr, "failed to parse user info metadata")
		}
		//设置选中
		key := logoSelectInfo.LogoSelected.TemplateTagSelected.TemplateTag
		if _, ok := mapTemplateTag[key]; ok {
			mapSelectColor[key] = logoSelectInfo.LogoSelected.TemplateTagSelected.SelectedIndex
		}
	}
	var templateTagNameList []string
	for templateTag, _ := range mapTemplateTag {
		templateTagNameList = append(templateTagNameList, templateTag)
	}
	productTemplateTags, err = l.svcCtx.AllModels.FsProductTemplateTags.GetListByTagNames(l.ctx, templateTagNameList, req.Limit, 1, "id DESC")
	if err != nil {
		logx.Error(err)
		return resp.SetStatusWithMessage(basic.CodeDbSqlErr, "failed to get template tags")
	}
	//资源id集合
	resourceIds := make([]string, 0, 5)
	for _, v := range productTemplateTags {
		resourceIds = append(resourceIds, s3url_to_s3id.GetS3ResourceIdFormUrl(*v.Cover))
	}
	//根据resourceUrls找到对应的元数据
	resourceMetadataList, err := l.svcCtx.AllModels.FsResource.FindAllByResourceIds(l.ctx, resourceIds)
	if err != nil {
		logx.Error(err)
		return resp.SetStatusWithMessage(basic.CodeDbSqlErr, "failed to get resource list")
	}
	mapResourceMetadata := make(map[string]map[string]interface{})
	for _, v := range resourceMetadataList {
		var metadata map[string]interface{}
		if v.Metadata != nil {
			_ = json.Unmarshal(*v.Metadata, &metadata)
		}
		mapResourceMetadata[*v.ResourceUrl] = metadata
	}
	list := make([]types.GetProductTemplateTagsRsp, 0, len(productTemplateTags))
	for _, templateInfo := range productTemplateTags {
		colors := make([]types.ColorsItem, 0, 10)
		SelectedIndex := 0
		for _, colorsSet := range mapTemplateTag[*templateInfo.TemplateTag] {
			if selectIndex, ok := mapSelectColor[*templateInfo.TemplateTag]; ok {
				SelectedIndex = selectIndex
			}
			colors = append(colors, types.ColorsItem{
				Color: colorsSet,
			})
		}
		list = append(list, types.GetProductTemplateTagsRsp{
			Id:            templateInfo.Id,
			TemplateTag:   *templateInfo.TemplateTag,
			Cover:         *templateInfo.Cover,
			CoverMetadata: mapResourceMetadata[*templateInfo.Cover],
			Colors:        colors,
			SelectedIndex: SelectedIndex,
		})
	}
	return resp.SetStatusWithMessage(basic.CodeOK, "success", list)
}
